##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "reStructuredText"

from zope import interface
from zope import component

from zope.pagetemplate.interfaces import IPageTemplate
from zope.publisher.browser import BrowserView

from interfaces import ITemplatedContentProvider


class TemplatedContentProvider( object ):
    interface.implements( ITemplatedContentProvider )

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def update(self):
        pass

    def render(self):
        return self.template()

class BaseView( TemplatedContentProvider, BrowserView ):

    def __call__(self):
        self.update()
        return self.render()

class MasterTemplatedContentProvider( TemplatedContentProvider ):


    def render(self):
        return self.master()




class MasterView( MasterTemplatedContentProvider, BrowserView ):

    def __call__(self):
        self.update()
        return self.render()



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: pagetemplate.py 70786 2006-10-18 14:33:55Z jukart $
"""
__docformat__ = "reStructuredText"

from zope import interface
from zope import component

from zope.pagetemplate.interfaces import IPageTemplate
from zope.publisher.browser import BrowserView



class RegisteredPageTemplate(object):


    def __init__(self, name=None):
        self.name = name

    def __call__(self, instance, *args, **keywords):
        import sys
        if self.name:
            template = component.getMultiAdapter(
                    (instance, instance.request), IPageTemplate, name=self.name)
        else:
            template = component.getMultiAdapter(
                    (instance, instance.request), IPageTemplate)

        # for viewtemplate compatiblity
        if isinstance(template, tuple):
            template = template[1]

        return template(instance, *args, **keywords)

    def __get__(self, instance, type):
        return BoundRegisteredPageTemplate(self, instance)


class BoundRegisteredPageTemplate(object):
    def __init__(self, pt, ob):
        #import pdb; pdb.set_trace()
        object.__setattr__(self, 'im_func', pt)
        object.__setattr__(self, 'im_self', ob)

    def __call__(self, *args, **kw):

        if self.im_self is None:
            im_self, args = args[0], args[1:]
        else:
            im_self = self.im_self
        return self.im_func(im_self, *args, **kw)

    def __setattr__(self, name, v):
        raise AttributeError("Can't set attribute", name)

    def __repr__(self):
        return "<BoundRegisteredPageTemplateFile of %r>" % self.im_self




##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = 'restructuredtext'

import unittest

from zope.testing import doctest
from zope.testing.doctestunit import DocFileSuite
from zope.app.testing import setup


def setUp(test):
    root = setup.placefulSetUp(site=True)
    test.globs['root'] = root


def tearDown(test):
    setup.placefulTearDown()


def test_suite():
    return unittest.TestSuite((
        DocFileSuite('./README.txt',
            setUp=setUp, tearDown=tearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            ),
        ))


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



import os
from setuptools import setup, find_packages

setup(
    name="z3c.pluggabletemplates",
    version="0.2",
    install_requires=['setuptools'],
    packages=find_packages('src'),
    package_dir= {'':'src'},

    namespace_packages=['z3c'],
    package_data = {
    '': ['*.txt', '*.zcml'],
    },


    # metadata for upload to PyPi
    author='Kevin Smith',
    author_email='kevin@mcweekly.com',
    url="http://code.google.com/p/pluggabletemplates/",
    description="Allows seperation of view code from skin templates like z3c.viewtemplate, but also allows multiple templates to be plugged into your view code.",
    long_description=(
    open('./src/z3c/pluggabletemplates/README.txt').read()
    + '\n' +
    open('./src/z3c/pluggabletemplates/CHANGES.txt').read()),
    license='ZPL 2.1',
    keywords="zope zope3 template",
    zip_safe=False,
    classifiers = ['Framework :: Zope3'],
    )



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "reStructuredText"

import os

from zope import interface
from zope import schema

from zope.component import zcml

from zope.configuration.exceptions import ConfigurationError
import zope.configuration.fields

from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.pagetemplate.interfaces import IPageTemplate
from zope.configuration.fields import GlobalObject

from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile


from zope.i18nmessageid import MessageFactory
_ = MessageFactory( 'zope' )


class IPluggableTemplatesDirective( interface.Interface ):
    """Parameters for the template directive."""


    for_ = GlobalObject(
            title = _( u'View' ),
            description = _( u'The view for which the template should be used' ),
            required = False,
            default=interface.Interface,
            )

    layer = GlobalObject(
            title = _( u'Layer' ),
            description = _( u'The layer for which the template should be used' ),
            required = False,
            default=IDefaultBrowserLayer,
            )


class IPluggableTemplate( interface.Interface ):

    file = zope.configuration.fields.Path(
            title=_( "Content-generating template." ),
            description=_( "Refers to a file containing a page template (should "
                          "end in extension ``.pt`` or ``.html``)." ),
            required=False,
            )

    name = schema.TextLine(
            title = _( u'Name' ),
            description = _( u"""
                The name to be used.
                Allows named adapter lookups so multiple templates can be assigned to one view.
                """ ),
            required = False,
            default = u'',
            )

    contentType = schema.BytesLine(
        title = _( u'Content Type' ),
        description=_( u'The content type identifies the type of data.' ),
        default='text/html',
        required=False,
        )

    layer = GlobalObject(
            title = _( u'Layer' ),
            description = _( u'The layer for which the template should be used' ),
            required = False,
            default=IDefaultBrowserLayer,
            )

class TemplateFactory( object ):

    def __init__( self, filename, contentType ):
        self.filename = filename
        self.contentType = contentType

    def __call__( self, view, request ):
        template = ViewPageTemplateFile( self.filename,
                                        content_type=self.contentType )
        return template



def templateDirective( _context,
                      file,
                      name,
                      for_=interface.Interface,
                      layer=IDefaultBrowserLayer,
                      contentType='text/html', **kwargs
                      ):
    # Make sure that the template exists
    file = os.path.abspath( str( _context.path( file ) ) )
    if not os.path.isfile( file ):
        raise ConfigurationError( "No such file", file )

    factory = TemplateFactory( file, contentType )

    zcml.adapter( _context, ( factory, ), IPageTemplate, ( for_, layer ), name=name )

class PluggableTemplatesDirective( object ):

    def __init__( self, _context, for_, layer=IDefaultBrowserLayer ):
        self._context = _context
        self.for_ = for_
        self.layer = layer

    def template( self, _context, file, name, contentType='text/html', layer=None, *args, **kw ):
        #import pdb; pdb.set_trace()
        file = os.path.abspath( str( self._context.path( file ) ) )

        if not layer:
            layer = self.layer

        return templateDirective( self._context, file, name, self.for_, layer, contentType )

    def __call__( self ):
        pass
